@@ -23,6 +23,8 @@
 
 #include "KCupsPasswordDialog.h"
 
+#include <config.h>
+
 #include <QCoreApplication>
 #include <QStringBuilder>
 #include <QDBusConnection>
@@ -174,6 +176,50 @@ KCupsConnection::KCupsConnection(QObject *parent) :
                     this,
                     SIGNAL(jobCompleted(QString,QString,QString,uint,QString,bool,uint,uint,QString,QString,uint)));
 
+    // This signal is needed since the cups registration thing
+    // doesn't emit printerAdded when we add a printer class
+    // This is emitted when a printer/queue is changed
+    QDBusConnection::systemBus().connect(QLatin1String(""),
+                                         QLatin1String("/com/redhat/PrinterSpooler"),
+                                         QLatin1String("com.redhat.PrinterSpooler"),
+                                         QLatin1String("PrinterAdded"),
+                                         this,
+                                         SIGNAL(rhPrinterAdded(QString)));
+
+    // This signal is needed since the cups registration thing
+    // sometimes simple stops working... don't ask me why
+    // This is emitted when a printer/queue is changed
+    QDBusConnection::systemBus().connect(QLatin1String(""),
+                                         QLatin1String("/com/redhat/PrinterSpooler"),
+                                         QLatin1String("com.redhat.PrinterSpooler"),
+                                         QLatin1String("QueueChanged"),
+                                         this,
+                                         SIGNAL(rhQueueChanged(QString)));
+
+    // This signal is needed since the cups registration thing
+    // doesn't emit printerRemoved when we add a printer class
+    // This is emitted when a printer/queue is changed
+    QDBusConnection::systemBus().connect(QLatin1String(""),
+                                         QLatin1String("/com/redhat/PrinterSpooler"),
+                                         QLatin1String("com.redhat.PrinterSpooler"),
+                                         QLatin1String("PrinterRemoved"),
+                                         this,
+                                         SIGNAL(rhPrinterRemoved(QString)));
+
+    QDBusConnection::systemBus().connect(QLatin1String(""),
+                                         QLatin1String("/com/redhat/PrinterSpooler"),
+                                         QLatin1String("com.redhat.PrinterSpooler"),
+                                         QLatin1String("JobQueuedLocal"),
+                                         this,
+                                         SIGNAL(rhJobQueuedLocal(QString,uint,QString)));
+
+    QDBusConnection::systemBus().connect(QLatin1String(""),
+                                         QLatin1String("/com/redhat/PrinterSpooler"),
+                                         QLatin1String("com.redhat.PrinterSpooler"),
+                                         QLatin1String("JobStartedLocal"),
+                                         this,
+                                         SIGNAL(rhJobStartedLocal(QString,uint,QString)));
+
     // Starts this thread
     start();
 }
@@ -181,6 +227,9 @@ KCupsConnection::KCupsConnection(QObject *parent) :
 KCupsConnection::~KCupsConnection()
 {
     m_instance = 0;
+    if (m_subscriptionId != -1) {
+        cancelDBusSubscription();
+    }
     m_renewTimer->deleteLater();
     m_passwordDialog->deleteLater();
 
@@ -199,7 +248,8 @@ void KCupsConnection::run()
     // Creates the timer that will renew the DBus subscription
     m_renewTimer = new QTimer;
     m_renewTimer->setInterval(RENEW_INTERVAL);
-    connect(m_renewTimer, SIGNAL(timeout()), this, SLOT(renewDBusSubscription()));
+    m_renewTimer->moveToThread(this);
+    connect(m_renewTimer, SIGNAL(timeout()), this, SLOT(renewDBusSubscription()), Qt::DirectConnection);
 
     m_inited = true;
     exec();
@@ -298,6 +348,7 @@ int KCupsConnection::createDBusSubscription(const QStringList &events)
     foreach (const QString &event, events) {
         if (!currentEvents.contains(event)) {
             equal = false;
+            break;
         }
     }
 
@@ -321,8 +372,6 @@ int KCupsConnection::createDBusSubscription(const QStringList &events)
         cancelDBusSubscription();
     }
 
-    currentEvents << events;
-
     // Canculates the new events
     renewDBusSubscription();
 
@@ -358,6 +407,7 @@ int KCupsConnection::renewDBusSubscription(int subscriptionId, int leaseDuration
     int ret = -1;
 
     if (!readyToStart()) {
+        kWarning() << "Tryied to run on the wrong thread";
         return subscriptionId; // This is not intended to be used in the gui thread
     }
 
@@ -365,7 +415,6 @@ int KCupsConnection::renewDBusSubscription(int subscriptionId, int leaseDuration
     do {
         ipp_t *request;
         ipp_op_e operation;
-        ret = subscriptionId;
 
         // check if we have a valid subscription ID
         if (subscriptionId >= 0) {
@@ -405,16 +454,30 @@ int KCupsConnection::renewDBusSubscription(int subscriptionId, int leaseDuration
         response = cupsDoRequest(CUPS_HTTP_DEFAULT, request, "/");
     } while (retry("/"));
 
-    if (ret < 0 &&
-        response != NULL &&
-        response->request.status.status_code <= IPP_OK_CONFLICT) {
-        ipp_attribute_t *attr = NULL;
-        if ((attr = ippFindAttribute(response, "notify-subscription-id",
-                                     IPP_TAG_INTEGER)) == NULL) {
-            kDebug() << "No notify-subscription-id in response!";
+#ifdef HAVE_CUPS_1_6
+    if (response && ippGetStatusCode(response) == IPP_OK) {
+#else
+    if (response && response->request.status.status_code == IPP_OK) {
+#endif // HAVE_CUPS_1_6
+        ipp_attribute_t *attr;
+        if (subscriptionId >= 0) {
+            // Request was ok, just return the current subscription
+            ret = subscriptionId;
+        } else if ((attr = ippFindAttribute(response,
+                                            "notify-subscription-id",
+                                            IPP_TAG_INTEGER)) == NULL) {
+            kWarning() << "No notify-subscription-id in response!";
+            ret = -1;
         } else {
+#ifdef HAVE_CUPS_1_6
+            ret = ippGetInteger(attr, 0);
+#else
             ret = attr->values[0].integer;
+#endif // HAVE_CUPS_1_6
         }
+    } else {
+        kWarning() << "Request failed" << lastError();
+        ret = -1;
     }
 
     ippDelete(response);
@@ -459,9 +522,13 @@ void KCupsConnection::renewDBusSubscription()
 {
     // check if we have a valid subscription ID
     kDebug() << m_subscriptionId;
+
     if (m_subscriptionId >= 0) {
-        renewDBusSubscription(m_subscriptionId, SUBSCRIPTION_DURATION);
-    } else {
+        m_subscriptionId = renewDBusSubscription(m_subscriptionId, SUBSCRIPTION_DURATION);
+    }
+
+    // The above request might fail if the subscription was cancelled
+    if (m_subscriptionId < 0) {
         QStringList currentEvents;
         foreach (const QStringList &registeredEvents, m_requestedDBusEvents) {
             currentEvents << registeredEvents;
@@ -581,6 +648,40 @@ ReturnArguments KCupsConnection::parseIPPVars(ipp_t *response, int group_tag, bo
     ipp_attribute_t *attr;
     ReturnArguments ret;
 
+#ifdef HAVE_CUPS_1_6
+    QVariantHash destAttributes;
+    for (attr = ippFirstAttribute(response); attr != NULL; attr = ippNextAttribute(response)) {
+        // We hit an attribute sepparator
+        if (ippGetName(attr) == NULL) {
+            ret << destAttributes;
+            destAttributes.clear();
+            continue;
+        }
+
+        // Skip leading attributes until we hit a a group which can be a printer, job...
+        if (ippGetGroupTag(attr) != group_tag ||
+                (ippGetValueTag(attr) != IPP_TAG_INTEGER &&
+                 ippGetValueTag(attr) != IPP_TAG_ENUM &&
+                 ippGetValueTag(attr) != IPP_TAG_BOOLEAN &&
+                 ippGetValueTag(attr) != IPP_TAG_TEXT &&
+                 ippGetValueTag(attr) != IPP_TAG_TEXTLANG &&
+                 ippGetValueTag(attr) != IPP_TAG_LANGUAGE &&
+                 ippGetValueTag(attr) != IPP_TAG_NAME &&
+                 ippGetValueTag(attr) != IPP_TAG_NAMELANG &&
+                 ippGetValueTag(attr) != IPP_TAG_KEYWORD &&
+                 ippGetValueTag(attr) != IPP_TAG_RANGE &&
+                 ippGetValueTag(attr) != IPP_TAG_URI)) {
+            continue;
+        }
+
+        // Add a printer description attribute...
+        destAttributes[QString::fromUtf8(ippGetName(attr))] = ippAttrToVariant(attr);
+    }
+
+    if (!destAttributes.isEmpty()) {
+        ret << destAttributes;
+    }
+#else
     for (attr = response->attrs; attr != NULL; attr = attr->next) {
        /*
         * Skip leading attributes until we hit a a group which can be a printer, job...
@@ -635,6 +736,8 @@ ReturnArguments KCupsConnection::parseIPPVars(ipp_t *response, int group_tag, bo
             break;
         }
     }
+#endif // HAVE_CUPS_1_6
+
     return ret;
 }
 
@@ -667,6 +770,54 @@ ipp_t* KCupsConnection::ippNewDefaultRequest(const QString &name, bool isClass,
 QVariant KCupsConnection::ippAttrToVariant(ipp_attribute_t *attr)
 {
     QVariant ret;
+#ifdef HAVE_CUPS_1_6
+    switch (ippGetValueTag(attr)) {
+    case IPP_TAG_INTEGER:
+    case IPP_TAG_ENUM:
+        if (ippGetCount(attr) == 1) {
+            ret = ippGetInteger(attr, 0);
+        } else {
+            QList<int> values;
+            for (int i = 0; i < ippGetCount(attr); ++i) {
+                values << ippGetInteger(attr, i);
+            }
+            ret = qVariantFromValue(values);
+        }
+        break;
+    case IPP_TAG_BOOLEAN:
+        if (ippGetCount(attr)== 1) {
+            ret = ippGetBoolean(attr, 0);
+        } else {
+            QList<bool> values;
+            for (int i = 0; i < ippGetCount(attr); ++i) {
+                values << ippGetBoolean(attr, i);
+            }
+            ret = qVariantFromValue(values);
+        }
+        break;
+    case IPP_TAG_RANGE:
+    {
+        QVariantList values;
+        for (int i = 0; i < ippGetCount(attr); ++i) {
+            int rangeUpper;
+            values << ippGetRange(attr, i, &rangeUpper);
+            values << rangeUpper;
+        }
+        ret = values;
+    }
+        break;
+    default:
+        if (ippGetCount(attr)== 1) {
+            ret = QString::fromUtf8(ippGetString(attr, 0, NULL));
+        } else {
+            QStringList values;
+            for (int i = 0; i < ippGetCount(attr); ++i) {
+                values << QString::fromUtf8(ippGetString(attr, i, NULL));
+            }
+            ret = values;
+        }
+    }
+#else
     switch (attr->value_tag) {
     case IPP_TAG_INTEGER:
     case IPP_TAG_ENUM:
@@ -712,6 +863,7 @@ QVariant KCupsConnection::ippAttrToVariant(ipp_attribute_t *attr)
             ret = values;
         }
     }
+#endif // HAVE_CUPS_1_6
     return ret;
 }
 
@@ -803,11 +955,13 @@ const char * password_cb(const char *prompt, http_t *http, const char *method, c
     bool wrongPassword = password_retries > 1;
 
     // This will block this thread until exec is not finished
+    kDebug() << password_retries;
     QMetaObject::invokeMethod(passwordDialog,
                               "exec",
                               Qt::BlockingQueuedConnection,
                               Q_ARG(QString, QString::fromUtf8(cupsUser())),
                               Q_ARG(bool, wrongPassword));
+    kDebug() << passwordDialog->accepted();
 
     // The password dialog has just returned check the result
     // method that returns QDialog enums
