@@ -1,6 +1,6 @@
 /***************************************************************************
  *   Copyright (C) 2010 by Daniel Nicoletti                                *
- *   dantti85-pk@yahoo.com.br                                              *
+ *   dantti12@gmail.com                                                    *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -26,6 +26,7 @@
 
 #include <KCupsRequest.h>
 #include <KCupsPrinter.h>
+#include <NoSelectionRectDelegate.h>
 
 #include <QPainter>
 #include <QToolBar>
@@ -45,9 +46,22 @@ PrintQueueUi::PrintQueueUi(const KCupsPrinter &printer, QWidget *parent) :
     ui(new Ui::PrintQueueUi),
     m_destName(printer.name()),
     m_preparingMenu(false),
+    m_printerPaused(false),
     m_lastState(0)
 {
     ui->setupUi(mainWidget());
+
+    // since setupUi needs to setup on the mainWidget()
+    // we need to manually connect the buttons
+    connect(ui->cancelJobPB, SIGNAL(clicked()), this, SLOT(cancelJob()));
+    connect(ui->holdJobPB, SIGNAL(clicked()), this, SLOT(holdJob()));
+    connect(ui->resumeJobPB, SIGNAL(clicked()), this, SLOT(resumeJob()));
+
+    connect(ui->pausePrinterPB, SIGNAL(clicked()), this, SLOT(pausePrinter()));
+    connect(ui->configurePrinterPB, SIGNAL(clicked()), this, SLOT(configurePrinter()));
+
+    connect(ui->whichJobsCB, SIGNAL(currentIndexChanged(int)), this, SLOT(whichJobsIndexChanged(int)));
+
     // Needed so we have our dialog size saved
     setAttribute(Qt::WA_DeleteOnClose);
 
@@ -84,11 +98,14 @@ PrintQueueUi::PrintQueueUi(const KCupsPrinter &printer, QWidget *parent) :
     m_model = new PrintQueueModel(printer.name(), winId(), this);
     connect(m_model, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
             this, SLOT(updateButtons()));
+    connect(m_model, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
+            this, SLOT(update()));
     m_proxyModel = new PrintQueueSortFilterProxyModel(this);
     m_proxyModel->setSourceModel(m_model);
     m_proxyModel->setDynamicSortFilter(true);
 
     ui->jobsView->setModel(m_proxyModel);
+    ui->jobsView->setItemDelegate(new NoSelectionRectDelegate(this));
     // sort by status column means the jobs will be sorted by the queue order
     ui->jobsView->sortByColumn(PrintQueueModel::ColStatus, Qt::AscendingOrder);
     connect(ui->jobsView->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
@@ -126,29 +143,54 @@ PrintQueueUi::PrintQueueUi(const KCupsPrinter &printer, QWidget *parent) :
         header->hideSection(PrintQueueModel::ColSize);
     }
 
-    // This is emitted when a printer/queue is changed
-    QDBusConnection::systemBus().connect(QLatin1String(""),
-                                         QLatin1String("/com/redhat/PrinterSpooler"),
-                                         QLatin1String("com.redhat.PrinterSpooler"),
-                                         QLatin1String("QueueChanged"),
-                                         this,
-                                         SLOT(updatePrinter(QString)));
+    // This is emitted when a printer is modified
+    connect(KCupsConnection::global(),
+            SIGNAL(printerModified(QString,QString,QString,uint,QString,bool)),
+            this,
+            SLOT(updatePrinter(QString,QString,QString,uint,QString,bool)));
 
-    // This is emitted when a printer is added
-    QDBusConnection::systemBus().connect(QLatin1String(""),
-                                         QLatin1String("/com/redhat/PrinterSpooler"),
-                                         QLatin1String("com.redhat.PrinterSpooler"),
-                                         QLatin1String("PrinterAdded"),
-                                         this,
-                                         SLOT(updatePrinter(QString)));
+    // This is emitted when a printer has it's state changed
+    connect(KCupsConnection::global(),
+            SIGNAL(printerStateChanged(QString,QString,QString,uint,QString,bool)),
+            this,
+            SLOT(updatePrinter(QString,QString,QString,uint,QString,bool)));
+
+    // This is emitted when a printer is stopped
+    connect(KCupsConnection::global(),
+            SIGNAL(printerStopped(QString,QString,QString,uint,QString,bool)),
+            this,
+            SLOT(updatePrinter(QString,QString,QString,uint,QString,bool)));
+
+    // This is emitted when a printer is restarted
+    connect(KCupsConnection::global(),
+            SIGNAL(printerRestarted(QString,QString,QString,uint,QString,bool)),
+            this,
+            SLOT(updatePrinter(QString,QString,QString,uint,QString,bool)));
+
+    // This is emitted when a printer is shutdown
+    connect(KCupsConnection::global(),
+            SIGNAL(printerShutdown(QString,QString,QString,uint,QString,bool)),
+            this,
+            SLOT(updatePrinter(QString,QString,QString,uint,QString,bool)));
 
     // This is emitted when a printer is removed
-    QDBusConnection::systemBus().connect(QLatin1String(""),
-                                         QLatin1String("/com/redhat/PrinterSpooler"),
-                                         QLatin1String("com.redhat.PrinterSpooler"),
-                                         QLatin1String("PrinterRemoved"),
-                                         this,
-                                         SLOT(updatePrinter(QString)));
+    connect(KCupsConnection::global(),
+            SIGNAL(printerDeleted(QString,QString,QString,uint,QString,bool)),
+            this,
+            SLOT(updatePrinter(QString,QString,QString,uint,QString,bool)));
+
+    // This is emitted when a printer/queue is changed
+    // Deprecated stuff that works better than the above
+    connect(KCupsConnection::global(), SIGNAL(rhPrinterAdded(QString)),
+            this, SLOT(updatePrinter(QString)));
+
+    connect(KCupsConnection::global(), SIGNAL(rhPrinterRemoved(QString)),
+            this, SLOT(updatePrinter(QString)));
+
+    connect(KCupsConnection::global(), SIGNAL(rhQueueChanged(QString)),
+            this, SLOT(updatePrinter(QString)));
+
+    createSubscription();
 
     updatePrinter(m_destName);
 
@@ -177,6 +219,7 @@ int PrintQueueUi::columnCount(const QModelIndex &parent) const
 
 void PrintQueueUi::setState(int state, const QString &message)
 {
+    kDebug() << state << message;
     if (state != m_lastState ||
         ui->printerStatusMsgL->text() != message) {
         // save the last state so the ui doesn't need to keep updating
@@ -318,9 +361,9 @@ void PrintQueueUi::showHeaderContextMenu(const QPoint &point)
     }
 }
 
-
 void PrintQueueUi::updatePrinter(const QString &printer)
 {
+    kDebug() << printer << m_destName;
     if (printer != m_destName) {
         // It was another printer that changed
         return;
@@ -333,13 +376,28 @@ void PrintQueueUi::updatePrinter(const QString &printer)
     attr << KCUPS_PRINTER_STATE_MESSAGE;
 
     KCupsRequest *request = new KCupsRequest;
-    request->getPrinterAttributes(printer, m_isClass, attr);
     connect(request, SIGNAL(finished()), this, SLOT(getAttributesFinished()));
+    request->getPrinterAttributes(printer, m_isClass, attr);
+}
+
+void PrintQueueUi::updatePrinter(const QString &text, const QString &printerUri, const QString &printerName, uint printerState, const QString &printerStateReasons, bool printerIsAcceptingJobs)
+{
+    // REALLY? all these parameters just to say foo was added??
+    Q_UNUSED(text)
+    Q_UNUSED(printerUri)
+    Q_UNUSED(printerState)
+    Q_UNUSED(printerStateReasons)
+    Q_UNUSED(printerIsAcceptingJobs)
+    kDebug() << printerName << printerStateReasons;
+
+    updatePrinter(printerName);
 }
 
 void PrintQueueUi::getAttributesFinished()
 {
     KCupsRequest *request = qobject_cast<KCupsRequest *>(sender());
+    kDebug() << request->hasError() << request->printers().isEmpty();
+
     if (request->hasError() || request->printers().isEmpty()) {
         // if cups stops we disable our queue
         setEnabled(false);
@@ -366,8 +424,6 @@ void PrintQueueUi::getAttributesFinished()
 
 void PrintQueueUi::update()
 {
-    m_model->updateModel();
-
     // Set window title
     if (m_model->rowCount()) {
         if (m_title.isNull()) {
@@ -380,6 +436,29 @@ void PrintQueueUi::update()
     }
 }
 
+void PrintQueueUi::createSubscription()
+{
+    KCupsRequest *request = new KCupsRequest;
+    connect(request, SIGNAL(finished()), this, SLOT(createSubscriptionFinished()));
+    QStringList events;
+    events << "printer-added";
+    events << "printer-deleted";
+    events << "printer-state-changed";
+    events << "printer-modified";
+    request->createDBusSubscription(events);
+}
+
+void PrintQueueUi::createSubscriptionFinished()
+{
+    KCupsRequest *request = qobject_cast<KCupsRequest*>(sender());
+    if (!request || request->hasError() || request->subscriptionId() < 0) {
+        // in case of an error probe the server again in 1.5 seconds
+        QTimer::singleShot(1000, this, SLOT(createSubscription()));
+    }
+
+    request->deleteLater();
+}
+
 void PrintQueueUi::updateButtons()
 {
     bool cancel, hold, release;
@@ -462,20 +541,22 @@ void PrintQueueUi::modifyJob(int action, const QString &destName)
     }
 }
 
-void PrintQueueUi::on_pausePrinterPB_clicked()
+void PrintQueueUi::pausePrinter()
 {
     // STOP and RESUME printer
     KCupsRequest *request = new KCupsRequest;
     if (m_printerPaused) {
+        kDebug() << m_destName << "m_printerPaused";
         request->resumePrinter(m_destName);
     } else {
+        kDebug() << m_destName << "NOT m_printerPaused";
         request->pausePrinter(m_destName);
     }
     request->waitTillFinished();
     request->deleteLater();
 }
 
-void PrintQueueUi::on_configurePrinterPB_clicked()
+void PrintQueueUi::configurePrinter()
 {
     QDBusMessage message;
     message = QDBusMessage::createMethodCall(QLatin1String("org.kde.ConfigurePrinter"),
@@ -486,37 +567,36 @@ void PrintQueueUi::on_configurePrinterPB_clicked()
     QDBusConnection::sessionBus().send(message);
 }
 
-void PrintQueueUi::on_cancelJobPB_clicked()
+void PrintQueueUi::cancelJob()
 {
     // CANCEL a job
     modifyJob(PrintQueueModel::Cancel);
 }
 
-void PrintQueueUi::on_holdJobPB_clicked()
+void PrintQueueUi::holdJob()
 {
     // HOLD a job
     modifyJob(PrintQueueModel::Hold);
 }
 
-void PrintQueueUi::on_resumeJobPB_clicked()
+void PrintQueueUi::resumeJob()
 {
     // RESUME a job
     modifyJob(PrintQueueModel::Release);
 }
 
-void PrintQueueUi::on_whichJobsCB_currentIndexChanged(int index)
+void PrintQueueUi::whichJobsIndexChanged(int index)
 {
-    int whichJobs = CUPS_WHICHJOBS_ACTIVE;
+    int whichJobs;
     switch (index) {
-    case 0:
-        whichJobs = CUPS_WHICHJOBS_ACTIVE;
-        break;
     case 1:
         whichJobs = CUPS_WHICHJOBS_COMPLETED;
         break;
     case 2:
         whichJobs = CUPS_WHICHJOBS_ALL;
         break;
+    default:
+        whichJobs = CUPS_WHICHJOBS_ACTIVE;
     }
     m_model->setWhichJobs(whichJobs);
 }
